@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import platform
 import signal
@@ -17,19 +19,14 @@ from ..typing import ASGIFramework
 from ..utils import (
     check_multiprocess_shutdown_event,
     load_application,
-    MustReloadException,
+    MustReloadError,
     observe_changes,
     raise_shutdown,
     repr_socket_addr,
     restart,
-    Shutdown,
+    ShutdownError,
 )
 
-try:
-    from socket import AF_UNIX
-except ImportError:
-    AF_UNIX = None
-
 
 async def _windows_signal_support() -> None:
     # See https://bugs.python.org/issue23057, to catch signals on
@@ -111,7 +108,6 @@ async def worker_serve(
             await asyncio.start_server(
                 _server_callback,
                 backlog=config.backlog,
-                loop=loop,
                 ssl=ssl_context,
                 sock=sock,
                 ssl_handshake_timeout=ssl_handshake_timeout,
@@ -125,9 +121,7 @@ async def worker_serve(
             sock = _share_socket(sock)
 
         servers.append(
-            await asyncio.start_server(
-                _server_callback, backlog=config.backlog, loop=loop, sock=sock
-            )
+            await asyncio.start_server(_server_callback, backlog=config.backlog, sock=sock)
         )
         bind = repr_socket_addr(sock.family, sock.getsockname())
         await config.log.info(f"Running on http://{bind} (CTRL + C to quit)")
@@ -146,9 +140,9 @@ async def worker_serve(
     try:
         gathered_tasks = asyncio.gather(*tasks)
         await gathered_tasks
-    except MustReloadException:
+    except MustReloadError:
         reload_ = True
-    except (Shutdown, KeyboardInterrupt):
+    except (ShutdownError, KeyboardInterrupt):
         pass
     finally:
         for server in servers:
@@ -157,7 +151,7 @@ async def worker_serve(
 
         try:
             await asyncio.sleep(config.graceful_timeout)
-        except (Shutdown, KeyboardInterrupt):
+        except (ShutdownError, KeyboardInterrupt):
             pass
 
         # Retrieve the Gathered Tasks Cancelled Exception, to
@@ -245,7 +239,7 @@ def _cancel_all_tasks(loop: asyncio.AbstractEventLoop) -> None:
 
     for task in tasks:
         task.cancel()
-    loop.run_until_complete(asyncio.gather(*tasks, loop=loop, return_exceptions=True))
+    loop.run_until_complete(asyncio.gather(*tasks, return_exceptions=True))
 
     for task in tasks:
         if not task.cancelled() and task.exception() is not None:
