@@ -1,14 +1,23 @@
+from __future__ import annotations
+
 import importlib
 import importlib.util
 import logging
 import os
 import socket
-import ssl
 import stat
 import types
 import warnings
 from dataclasses import dataclass
-from ssl import SSLContext, VerifyFlags, VerifyMode
+from ssl import (
+    create_default_context,
+    OP_NO_COMPRESSION,
+    Purpose,
+    SSLContext,
+    TLSVersion,
+    VerifyFlags,
+    VerifyMode,
+)
 from time import time
 from typing import Any, AnyStr, Dict, List, Mapping, Optional, Tuple, Type, Union
 from wsgiref.handlers import format_date_time
@@ -46,6 +55,7 @@ class Config:
     _quic_bind: List[str] = []
     _quic_addresses: List[Tuple] = []
     _log: Optional[Logger] = None
+    _root_path: str = ""
 
     access_log_format = '%(h)s %(l)s %(l)s %(t)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
     accesslog: Union[logging.Logger, str, None] = None
@@ -60,6 +70,7 @@ class Config:
     dogstatsd_tags = ""
     errorlog: Union[logging.Logger, str, None] = "-"
     graceful_timeout: float = 3 * SECONDS
+    read_timeout: Optional[int] = None
     group: Optional[int] = None
     h11_max_incomplete_size = 16 * 1024 * BYTES
     h2_max_concurrent_streams = 100
@@ -71,10 +82,9 @@ class Config:
     logconfig: Optional[str] = None
     logconfig_dict: Optional[dict] = None
     logger_class = Logger
-    loglevel: Optional[str] = None
+    loglevel: str = "INFO"
     max_app_queue_size: int = 10
     pid_path: Optional[str] = None
-    root_path = ""
     server_names: List[str] = []
     shutdown_timeout = 60 * SECONDS
     ssl_handshake_timeout = 60 * SECONDS
@@ -87,7 +97,7 @@ class Config:
     verify_flags: Optional[VerifyFlags] = None
     verify_mode: Optional[VerifyMode] = None
     websocket_max_message_size = 16 * 1024 * 1024 * BYTES
-    websocket_ping_interval: Optional[int] = None
+    websocket_ping_interval: Optional[float] = None
     worker_class = "asyncio"
     workers = 1
 
@@ -136,18 +146,22 @@ class Config:
         else:
             self._quic_bind = value
 
+    @property
+    def root_path(self) -> str:
+        return self._root_path
+
+    @root_path.setter
+    def root_path(self, value: str) -> None:
+        self._root_path = value.rstrip("/")
+
     def create_ssl_context(self) -> Optional[SSLContext]:
         if not self.ssl_enabled:
             return None
 
-        context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
+        context = create_default_context(Purpose.CLIENT_AUTH)
         context.set_ciphers(self.ciphers)
-        cipher_opts = 0
-        for attr in ["OP_NO_SSLv2", "OP_NO_SSLv3", "OP_NO_TLSv1", "OP_NO_TLSv1_1"]:
-            if hasattr(ssl, attr):  # To be future proof
-                cipher_opts |= getattr(ssl, attr)
-        context.options |= cipher_opts  # RFC 7540 Section 9.2: MUST be TLS >=1.2
-        context.options |= ssl.OP_NO_COMPRESSION  # RFC 7540 Section 9.2.1: MUST disable compression
+        context.minimum_version = TLSVersion.TLSv1_2  # RFC 7540 Section 9.2: MUST be TLS >=1.2
+        context.options = OP_NO_COMPRESSION  # RFC 7540 Section 9.2.1: MUST disable compression
         context.set_alpn_protocols(self.alpn_protocols)
 
         if self.certfile is not None and self.keyfile is not None:
