@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 from typing import Awaitable, Callable, Optional, Tuple, Union
 
 from .h2 import H2Protocol
-from .h11 import H2CProtocolRequired, H2ProtocolAssumed, H11Protocol
+from .h11 import H2CProtocolRequiredError, H2ProtocolAssumedError, H11Protocol
 from ..config import Config
 from ..events import Event, RawData
 from ..typing import ASGIFramework, Context
@@ -58,7 +60,7 @@ class ProtocolWrapper:
     async def handle(self, event: Event) -> None:
         try:
             return await self.protocol.handle(event)
-        except H2ProtocolAssumed as error:
+        except H2ProtocolAssumedError as error:
             self.protocol = H2Protocol(
                 self.app,
                 self.config,
@@ -71,7 +73,7 @@ class ProtocolWrapper:
             await self.protocol.initiate()
             if error.data != b"":
                 return await self.protocol.handle(RawData(data=error.data))
-        except H2CProtocolRequired as error:
+        except H2CProtocolRequiredError as error:
             self.protocol = H2Protocol(
                 self.app,
                 self.config,
