@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from itertools import chain
 from typing import Awaitable, Callable, Optional, Tuple, Union
 
@@ -22,7 +24,7 @@ from ..typing import ASGIFramework, Context, H11SendableEvent
 STREAM_ID = 1
 
 
-class H2CProtocolRequired(Exception):
+class H2CProtocolRequiredError(Exception):
     def __init__(self, data: bytes, request: h11.Request) -> None:
         settings = ""
         headers = [(b":method", request.method), (b":path", request.target)]
@@ -38,7 +40,7 @@ class H2CProtocolRequired(Exception):
         self.settings = settings
 
 
-class H2ProtocolAssumed(Exception):
+class H2ProtocolAssumedError(Exception):
     def __init__(self, data: bytes) -> None:
         self.data = data
 
@@ -156,8 +158,8 @@ class H11Protocol:
                     await self._check_protocol(event)
                     await self._create_stream(event)
                 elif event is h11.PAUSED:
-                    await self.send(Updated())
                     await self.can_read.clear()
+                    await self.send(Updated())
                     await self.can_read.wait()
                 elif isinstance(event, h11.ConnectionClosed) or event is h11.NEED_DATA:
                     break
@@ -255,7 +257,6 @@ class H11Protocol:
         else:
             await self.can_read.set()
             await self.send(Closed())
-        await self._handle_events()
 
     async def _close_stream(self) -> None:
         if self.stream is not None:
@@ -285,6 +286,8 @@ class H11Protocol:
                     + [(b"connection", b"upgrade"), (b"upgrade", b"h2c")],
                 )
             )
-            raise H2CProtocolRequired(self.connection.trailing_data[0], event)
+            raise H2CProtocolRequiredError(self.connection.trailing_data[0], event)
         elif event.method == b"PRI" and event.target == b"*" and event.http_version == b"2.0":
-            raise H2ProtocolAssumed(b"PRI * HTTP/2.0\r\n\r\n" + self.connection.trailing_data[0])
+            raise H2ProtocolAssumedError(
+                b"PRI * HTTP/2.0\r\n\r\n" + self.connection.trailing_data[0]
+            )
