@@ -1,4 +1,7 @@
+from __future__ import annotations
+
 from enum import auto, Enum
+from io import BytesIO, StringIO
 from time import time
 from typing import Awaitable, Callable, List, Optional, Tuple, Union
 from urllib.parse import unquote
@@ -19,8 +22,21 @@ from wsproto.utilities import generate_accept_token, split_comma_header
 
 from .events import Body, Data, EndBody, EndData, Event, Request, Response, StreamClosed
 from ..config import Config
-from ..typing import ASGIFramework, Context
-from ..utils import build_and_validate_headers, suppress_body, UnexpectedMessage, valid_server_name
+from ..typing import (
+    ASGIFramework,
+    ASGISendEvent,
+    Context,
+    WebsocketAcceptEvent,
+    WebsocketResponseBodyEvent,
+    WebsocketResponseStartEvent,
+    WebsocketScope,
+)
+from ..utils import (
+    build_and_validate_headers,
+    suppress_body,
+    UnexpectedMessageError,
+    valid_server_name,
+)
 
 
 class ASGIWebsocketState(Enum):
@@ -33,7 +49,7 @@ class ASGIWebsocketState(Enum):
     HTTPCLOSED = auto()
 
 
-class FrameTooLarge(Exception):
+class FrameTooLargeError(Exception):
     pass
 
 
@@ -90,7 +106,7 @@ class Handshake:
 
         extensions: List[Extension] = [PerMessageDeflate()]
         accepts = None
-        if False and self.extensions is not None:
+        if self.extensions is not None:
             accepts = server_extensions_handshake(self.extensions, extensions)
 
         if accepts:
@@ -109,27 +125,29 @@ class Handshake:
 
 class WebsocketBuffer:
     def __init__(self, max_length: int) -> None:
-        self.value: Optional[Union[bytes, str]] = None
+        self.value: Optional[Union[BytesIO, StringIO]] = None
+        self.length = 0
         self.max_length = max_length
 
     def extend(self, event: Message) -> None:
         if self.value is None:
             if isinstance(event, TextMessage):
-                self.value = ""
+                self.value = StringIO()
             else:
-                self.value = b""
-        self.value += event.data
-        if len(self.value) > self.max_length:
-            raise FrameTooLarge()
+                self.value = BytesIO()
+        self.length += self.value.write(event.data)
+        if self.length > self.max_length:
+            raise FrameTooLargeError()
 
     def clear(self) -> None:
         self.value = None
+        self.length = 0
 
     def to_message(self) -> dict:
         return {
             "type": "websocket.receive",
-            "bytes": self.value if isinstance(self.value, bytes) else None,
-            "text": self.value if isinstance(self.value, str) else None,
+            "bytes": self.value.getvalue() if isinstance(self.value, BytesIO) else None,
+            "text": self.value.getvalue() if isinstance(self.value, StringIO) else None,
         }
 
 
@@ -152,8 +170,8 @@ class WSStream:
         self.closed = False
         self.config = config
         self.context = context
-        self.response: dict
-        self.scope: dict
+        self.response: WebsocketResponseStartEvent
+        self.scope: WebsocketScope
         self.send = send
         # RFC 8441 for HTTP/2 says use http or https, ASGI says ws or wss
         self.scheme = "wss" if ssl else "ws"
@@ -202,7 +220,7 @@ class WSStream:
                 self.app_put = await self.context.spawn_app(
                     self.app, self.config, self.scope, self.app_send
                 )
-                await self.app_put({"type": "websocket.connect"})
+                await self.app_put({"type": "websocket.connect"})  # type: ignore
         elif isinstance(event, (Body, Data)):
             self.connection.receive_data(event.data)
             await self._handle_events()
@@ -215,7 +233,7 @@ class WSStream:
                     code = CloseReason.ABNORMAL_CLOSURE.value
                 await self.app_put({"type": "websocket.disconnect", "code": code})
 
-    async def app_send(self, message: Optional[dict]) -> None:
+    async def app_send(self, message: Optional[ASGISendEvent]) -> None:
         if self.closed:
             # Allow app to finish after close
             return
@@ -264,14 +282,14 @@ class WSStream:
                 )
                 await self.send(EndData(stream_id=self.stream_id))
             else:
-                raise UnexpectedMessage(self.state, message["type"])
+                raise UnexpectedMessageError(self.state, message["type"])
 
     async def _handle_events(self) -> None:
         for event in self.connection.events():
             if isinstance(event, Message):
                 try:
                     self.buffer.extend(event)
-                except FrameTooLarge:
+                except FrameTooLargeError:
                     await self._send_wsproto_event(
                         CloseConnection(code=CloseReason.MESSAGE_TOO_BIG)
                     )
@@ -304,7 +322,7 @@ class WSStream:
         data = self.connection.send(event)
         await self.send(Data(stream_id=self.stream_id, data=data))
 
-    async def _accept(self, message: dict) -> None:
+    async def _accept(self, message: WebsocketAcceptEvent) -> None:
         self.state = ASGIWebsocketState.CONNECTED
         status_code, headers, self.connection = self.handshake.accept(message.get("subprotocol"))
         await self.send(
@@ -316,7 +334,7 @@ class WSStream:
         if self.config.websocket_ping_interval is not None:
             self.context.spawn(self._send_pings)
 
-    async def _send_rejection(self, message: dict) -> None:
+    async def _send_rejection(self, message: WebsocketResponseBodyEvent) -> None:
         body_suppressed = suppress_body("GET", self.response["status"])
         if self.state == ASGIWebsocketState.HANDSHAKE:
             headers = build_and_validate_headers(self.response["headers"])
